﻿-- ********************************************************************************
-- Broker Statistics
-- A Data Broker display for combat statistics.
-- By: Shenton
--
-- IconsFrame.lua
-- Icons selection frame.
-- ********************************************************************************

local A = _G["DataBrokerStatisticsGlobal"];

-- Globals to locals
local string = string;
local ceil = ceil;
local pairs = pairs;
local _G = _G;

-- GLOBALS: GetNumSpellTabs, GetSpellTabInfo, GetSpellBookItemInfo, GetSpellBookItemTexture
-- GLOBALS: GetFlyoutInfo, GetFlyoutSlotInfo, GetSpellTexture, GetMacroIcons, GetMacroItemIcons
-- GLOBALS: FauxScrollFrame_GetOffset, BrokerStatisticsPopupFrameScrollFrame, BrokerStatisticsPopupFrame
-- GLOBALS: FauxScrollFrame_Update, CreateFrame, UIParent, SystemFont_Med1, CreateFrame
-- GLOBALS: BrokerStatisticsPopupFrameButtonCancel, FauxScrollFrame_OnVerticalScroll, GetScreenWidth
-- GLOBALS: BrokerStatisticsPopupFrameButtonAccept, GameTooltip

-- ********************************************************************************
-- Variables & constants
-- ********************************************************************************

A.popupFrameIconsList = {};
A.popupFrameIconsListSearch = {};

-- ********************************************************************************
-- Functions
-- ********************************************************************************

--- Reset and add icons to the icons list table
function A:GetIconsList()
    A.popupFrameIconsList = {};
    A.popupFrameIconsList[1] = "INV_MISC_QUESTIONMARK";

    local index = 2;
    local numFlyouts = 0;

    for i=1,GetNumSpellTabs() do
        local tab, tabTex, offset, numSpells, _ = GetSpellTabInfo(i);
        local tabEnd = offset + numSpells;

        offset = offset + 1;

        for j=offset,tabEnd-1 do
            local spellType, ID = GetSpellBookItemInfo(j, "player");

            if ( spellType ~= "FUTURESPELL" ) then
                local spellTexture = string.upper(GetSpellBookItemTexture(j, "player"));

                if ( not string.match(spellTexture, "INTERFACE\\BUTTONS\\") ) then
                    A.popupFrameIconsList[index] = string.gsub(spellTexture, "INTERFACE\\ICONS\\", "");
                    index = index + 1;
                end
            end
            if ( spellType == "FLYOUT" ) then
                local _, _, numSlots, isKnown = GetFlyoutInfo(ID);

                if ( isKnown and numSlots > 0 ) then
                    for k=1,numSlots do
                        local spellID, overrideSpellID, isKnown = GetFlyoutSlotInfo(ID, k);

                        if ( isKnown ) then
                            A.popupFrameIconsList[index] = string.gsub(string.upper(GetSpellTexture(spellID)), "INTERFACE\\ICONS\\", ""); 
                            index = index + 1;
                        end
                    end
                end
            end
        end
    end

    GetLooseMacroIcons(A.popupFrameIconsList);
    GetLooseMacroItemIcons(A.popupFrameIconsList);
    GetMacroIcons(A.popupFrameIconsList);
    GetMacroItemIcons(A.popupFrameIconsList);
end

--- Icon search method
function A:IconSearch(text)
    A.popupFrameIconsListSearch = {};

    if ( not text or text == "" ) then return; end

    local gotOne;
    text = string.lower(text);

    for k,v in pairs(A.popupFrameIconsList) do
        if ( string.find(string.lower(v), text) ) then
            A.popupFrameIconsListSearch[#A.popupFrameIconsListSearch+1] = v;
            gotOne = 1;
        end
    end

    if ( not gotOne ) then
        A.popupFrameIconsListSearch[1] = "INV_MISC_QUESTIONMARK";
    end
end

--- Init the icon selection frame and update when scrolling (Blizzard guild bank name/icon selection frame)
function A:PopupScrollUpdate()
    local icons, icon, button, index, texture;
    local popupOffset = FauxScrollFrame_GetOffset(BrokerStatisticsPopupFrameScrollFrame);

    if ( #A.popupFrameIconsListSearch > 0 ) then
        icons = #A.popupFrameIconsListSearch;
    else
        icons = #A.popupFrameIconsList;
    end

    for i=1,64 do
        icon = _G["BrokerStatisticsPopupFrameButton"..i.."Icon"];
        button = _G["BrokerStatisticsPopupFrameButton"..i];
        index = (popupOffset * 8) + i;

        if ( index ) then
            if ( #A.popupFrameIconsListSearch > 0 ) then
                texture = A.popupFrameIconsListSearch[index];
            else
                texture = A.popupFrameIconsList[index];
            end
        else
            texture = nil;
        end

        if ( index <= icons and texture ) then
            if( type(texture) == "number" ) then
                icon:SetToFileData(texture);
                texture = icon:GetTexture();
                texture = string.gsub(string.upper(texture), "INTERFACE\\ICONS\\", "");
            else
                icon:SetTexture("INTERFACE\\ICONS\\"..texture);
            end
            button.textureName = texture;
            button:Show();
        else
            icon:SetTexture("");
            button.textureName = nil;
            button:Hide();
        end

        if ( BrokerStatisticsPopupFrame.selectedIcon ) then
            if ( index == BrokerStatisticsPopupFrame.selectedIcon ) then
                button:SetChecked(1);
            else
                button:SetChecked(nil);
            end
        elseif ( BrokerStatisticsPopupFrame.currentTexture == texture ) then
            button:SetChecked(1);
            BrokerStatisticsPopupFrame.selectedIcon = index;
        else
            button:SetChecked(nil);
        end
    end

    if ( not BrokerStatisticsPopupFrame.selectedIcon ) then
        for i=1, icons do
            if ( #A.popupFrameIconsListSearch > 0 ) then
                texture = A.popupFrameIconsListSearch[index];
            else
                texture = A.popupFrameIconsList[index];
            end

            if ( BrokerStatisticsPopupFrame.currentTexture == texture ) then
                BrokerStatisticsPopupFrame.selectedIcon = i;
                break;
            end
        end
    end

    FauxScrollFrame_Update(BrokerStatisticsPopupFrameScrollFrame, ceil(icons / 8) , 8, 36);
end

--- When an icon is clicked
function A:PopupScrollButtonClick(self)
    local offset = FauxScrollFrame_GetOffset(BrokerStatisticsPopupFrameScrollFrame);
    local index = (offset * 8) + self:GetID();
    BrokerStatisticsPopupFrame.selectedIcon = index;
    BrokerStatisticsPopupFrame.selectedIconTexture = self.textureName;
    A:PopupScrollUpdate();
end

-- Frame
function A:IconsFrame()
    local f = CreateFrame("Frame", "BrokerStatisticsPopupFrame", UIParent);
    f:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        tile = true, tileSize = 32, edgeSize = 32,
        insets = { left = 8, right = 8, top = 8, bottom = 8 }});
    f:SetClampedToScreen(1);
    f:EnableMouse(1);
    f:SetFrameStrata("FULLSCREEN_DIALOG");
    f:Hide();
    f:SetSize(416, 500);

    -- Text
    f.text = f:CreateFontString("BrokerStatisticsPopupFrameText");
    f.text:SetFontObject(SystemFont_Med1);
    f.text:SetText(A.L["Search"]);
    f.text:SetPoint("TOPLEFT", f, 16, -16);

    -- Search box
    f.search = CreateFrame("EditBox", "BrokerStatisticsPopupFrameSearchBox", f, "SearchBoxTemplate");
    f.search:SetSize(256, 48);
    f.search:SetPoint("TOPLEFT", f, 20, -20);
    f.search:SetScript("OnTextChanged", function(self)
        local text = self:GetText();

        A:IconSearch(text);
        A:PopupScrollUpdate();
    end);

    -- Scroll
    f.scroll = CreateFrame("ScrollFrame", "BrokerStatisticsPopupFrameScrollFrame", f, "ListScrollFrameTemplate");
    f.scroll:SetSize(370, 350);
    f.scroll:SetPoint("RIGHT", f, "RIGHT", -34, -34);
    f.scroll:SetScript("OnVerticalScroll", function(self, offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, 36, A.PopupScrollUpdate);
    end);

    -- Icons buttons
    f.button1 = CreateFrame("CheckButton", "BrokerStatisticsPopupFrameButton1", f, "PopupButtonTemplate");
    f.button1:SetID(1);
    f.button1:SetPoint("TOPLEFT", f, "TOPLEFT", 16, -110);
    f.button1:SetScript("OnClick", function(self) A:PopupScrollButtonClick(self); end);
    f.button1:HookScript("OnEnter", function(self)
        if ( self:GetRight() >= ( GetScreenWidth() / 2 ) ) then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT");
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
        end

        GameTooltip:AddLine(self.textureName);
        GameTooltip:Show();
    end);
    f.button1:HookScript("OnLeave", function(self)
        GameTooltip:Hide();
    end);

    local button, row = 2, 1;
    while ( button <= 64 ) do
        if ( row == 8 ) then
            f["button"..button] = CreateFrame("CheckButton", "BrokerStatisticsPopupFrameButton"..button, _G["BrokerStatisticsPopupFrameButton"..button - 4], "PopupButtonTemplate");
            f["button"..button]:SetID(button);
            f["button"..button]:SetPoint("TOPLEFT", _G["BrokerStatisticsPopupFrameButton"..button - 8], "BOTTOMLEFT", 0, -8);
            f["button"..button]:SetScript("OnClick", function(self) A:PopupScrollButtonClick(self); end);
            f["button"..button]:HookScript("OnEnter", function(self)
                if ( self:GetRight() >= ( GetScreenWidth() / 2 ) ) then
                    GameTooltip:SetOwner(self, "ANCHOR_LEFT");
                else
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
                end

                GameTooltip:AddLine(self.textureName);
                GameTooltip:Show();
            end);
            f["button"..button]:HookScript("OnLeave", function(self)
                GameTooltip:Hide();
            end);
            row = 1;
        else
            f["button"..button] = CreateFrame("CheckButton", "BrokerStatisticsPopupFrameButton"..button, _G["BrokerStatisticsPopupFrameButton"..button - 1], "PopupButtonTemplate");
            f["button"..button]:SetID(button);
            f["button"..button]:SetPoint("LEFT", _G["BrokerStatisticsPopupFrameButton"..button - 1], "RIGHT", 10, 0);
            f["button"..button]:SetScript("OnClick", function(self) A:PopupScrollButtonClick(self); end);
            f["button"..button]:HookScript("OnEnter", function(self)
                if ( self:GetRight() >= ( GetScreenWidth() / 2 ) ) then
                    GameTooltip:SetOwner(self, "ANCHOR_LEFT");
                else
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
                end

                GameTooltip:AddLine(self.textureName);
                GameTooltip:Show();
            end);
            f["button"..button]:HookScript("OnLeave", function(self)
                GameTooltip:Hide();
            end);
            row = row + 1;
        end

        button = button + 1;
    end

    -- Cancel button
    f.cancelButton = CreateFrame("Button", "BrokerStatisticsPopupFrameButtonCancel", f, "UIPanelButtonTemplate");
    f.cancelButton:SetSize(69, 22);
    f.cancelButton:SetText(A.L["Cancel"]);
    f.cancelButton:SetScript("OnClick", function()
        BrokerStatisticsPopupFrame:Hide();
        BrokerStatisticsPopupFrame.ldbObject = nil;
        BrokerStatisticsPopupFrame.selectedIcon = nil;
        --BrokerStatisticsPopupFrame.iconList = nil;
    end);
    f.cancelButton:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -16, 16);

    -- Accept button
    f.acceptButton = CreateFrame("Button", "BrokerStatisticsPopupFrameButtonAccept", BrokerStatisticsPopupFrameButtonCancel, "UIPanelButtonTemplate");
    f.acceptButton:SetSize(69, 22);
    f.acceptButton:SetText(A.L["Accept"]);
    f.acceptButton:SetScript("OnClick", function()
        if ( not BrokerStatisticsPopupFrame.selectedIcon ) then
            A:Message(A.L["No icon selected."], 1, 1);
        else
            local texture;

            BrokerStatisticsPopupFrame:Hide();
            texture = "INTERFACE\\ICONS\\"..BrokerStatisticsPopupFrame.selectedIconTexture;
            BrokerStatisticsPopupFrame.dbEdit.icon = texture;
            if ( BrokerStatisticsPopupFrame.ldbObject ) then
                BrokerStatisticsPopupFrame.ldbObject.icon = texture;
            end
            BrokerStatisticsPopupFrame.dbEdit = nil;
            BrokerStatisticsPopupFrame.ldbObject = nil;
            BrokerStatisticsPopupFrame.selectedIcon = nil;
            --BrokerStatisticsPopupFrame.iconList = nil;

            A:RefreshConfig();
        end
    end);
    f.acceptButton:SetPoint("RIGHT", BrokerStatisticsPopupFrameButtonCancel, "LEFT", -2, 0);

    -- Default button
    f.defaultButton = CreateFrame("Button", "BrokerStatisticsPopupFrameButtonDefault", BrokerStatisticsPopupFrameButtonAccept, "UIPanelButtonTemplate");
    f.defaultButton:SetSize(69, 22);
    f.defaultButton:SetText(A.L["Default"]);
    f.defaultButton:SetScript("OnClick", function()
        BrokerStatisticsPopupFrame:Hide();

        BrokerStatisticsPopupFrame.dbEdit.icon = A.defaultLdbIcon;
        if ( BrokerStatisticsPopupFrame.ldbObject ) then
            BrokerStatisticsPopupFrame.ldbObject.icon = A.defaultLdbIcon;
        end
        BrokerStatisticsPopupFrame.dbEdit = nil;
        BrokerStatisticsPopupFrame.ldbObject = nil;
        BrokerStatisticsPopupFrame.selectedIcon = nil;
        --BrokerStatisticsPopupFrame.iconList = nil;

        A:RefreshConfig();
    end);
    f.defaultButton:SetPoint("RIGHT", BrokerStatisticsPopupFrameButtonAccept, "LEFT", -2, 0);

    f:SetScript("OnShow", function()
        f.search:SetText("");
        A:GetIconsList();
    end);
end
